//Voice decompressor example
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

//I like these definitions
#define begin {
#define end   } 

//#define TableSize 1520 //refers to the following incl file
//Contains the packed 2-bit codes for syntehsis
//Generated by the program Make2code476.m
#include "LCD/lcd_lib.h"

unsigned char CurrentTable;

//reconstruction differentials 
//  PCMvalue[4] = {-78, -16, 16, 78};
volatile signed char PCMvalue[4] = {-20, -4, 4, 20};

volatile unsigned int outI, tableI;     //indexes
volatile unsigned char cycle ;  		//decode phase counter
volatile signed char out, lastout;		//output values
volatile unsigned char p1, p2, p3, p4;	//hold 4 differentials
volatile unsigned char packed	;		//byte containing 4 2-bit values	

volatile unsigned int TableSize = 0;

unsigned char output[16] = "                ";
unsigned char buttonDown0 = 0, buttonDown1 = 0, buttonDown2 = 0, buttonDown3 = 0, buttonDown4 = 0;
unsigned char metermode = 0;

float min, max, mean;
unsigned int meanCount;

unsigned char takeFMeasure = 0;

int Ain;   
float voltage;

uint32_t frequency;
unsigned char overflowCount = 0;
unsigned char frequencyMeasured = 0; //flag if frequency has been meausred
// 0 = no. 1 = synched, 2 = has been measured.

#define LOGIC_THRESHOLD 2.0
#define VREF 5.0

#include "voice.h"

//generate waveform at 7812 scamples/sec
ISR (TIMER2_OVF_vect)
begin 
	//compute next sample
	cycle = outI & 3;  // outI modulo 4
	if (cycle==0)      //do we need to unpack more data?
	{
        if (tableI<TableSize)  //end of stored wave?
        {
        	//unpack a table entry into 2-bit indexs
			// pgm_read_byte (address_short) 
			switch(CurrentTable)
			{
				case Volts_e:
					packed = pgm_read_byte(&Volts_wav[tableI]) ;
					break;
				case Point_e:
					packed = pgm_read_byte(&Point_wav[tableI]) ;
					break;
				case AllDigits_e:
					packed = pgm_read_byte(&AllDigits_wav[tableI]) ;
					break;
				case Ready_e:
					packed = pgm_read_byte(&Ready_wav[tableI]) ;
					break;
				case Beeplow_e:
					packed = pgm_read_byte(&Beeplow_wav[tableI]) ;
					break;
				case Beephigh_e:
					packed = pgm_read_byte(&Beephigh_wav[tableI]) ;
					break;
				case High_e:
					packed = pgm_read_byte(&High_wav[tableI]) ;
					break;
				case Low_e:
					packed = pgm_read_byte(&Low_wav[tableI]) ;
					break;
				case Mean_e:
					packed = pgm_read_byte(&Mean_wav[tableI]) ;
					break;
				case Samples_e:
					packed = pgm_read_byte(&Samples_wav[tableI]) ;
					break;
				case Over_e:
					packed = pgm_read_byte(&Over_wav[tableI]) ;
					break;
				case Breakfast_e:
					packed = pgm_read_byte(&Breakfast_wav[tableI]) ;
					break;
				case Hz_e:
					packed = pgm_read_byte(&Hz_wav[tableI]) ;
					break;
				case Nosignal_e:
					packed = pgm_read_byte(&Nosignal_wav[tableI]) ;
					break;
					
			}
					
        	//packed = DPCMAllDigits[tableI];
            p1 = (packed>>6) & 3 ;
           	p2 = (packed>>4) & 3 ;
            p3 = (packed>>2) & 3 ;
            p4 = (packed & 3); 
            tableI++ ; 
        } //end unpack table entry 
 		//compute the output and send to PWM
        out = lastout + PCMvalue[p1] - (lastout>>3) ;  	
 	}
 	else if (cycle==1)    //don't need to unpack yet--just ouput
 		out = lastout + PCMvalue[p2] - (lastout>>3) ;
  	else if (cycle==2)
  		out = lastout + PCMvalue[p3] - (lastout>>3) ; 
  	else if (cycle==3) 
  		out = lastout + PCMvalue[p4] - (lastout>>3) ;

	//update outputs
	OCR0A = out + 128;
    lastout = out;
    outI++;
	//at end, turn off TCCRO
	if (tableI>=TableSize) 
	{
		TCCR0B = 0;	  
		say_next();
	}
end //ISR



void init_lcd(void) 
begin
	LCDinit();	//initialize the display
	LCDcursorOFF();
	LCDclr();				//clear the display
	LCDGotoXY(0,0);
	LCDstring("Yo. Ima LCD.", 12);
	LCDGotoXY(0,1);
	LCDstring("Pick a Pgm, Yo.", 15);
end	

void checkButtons()
{
	//Probe Button
	//very basic anti-bounce, as we only care about the rising edge
	if(!(PINB & 0x01)) //button is down
	{
		//LCDsendChar('*');
		if(buttonDown0 == 0) //and not down before
			buttonDown0++;
		else if(buttonDown0 == 1) //and down once
		{
			switch(metermode) //Initial Button Press
			{
				case 1: //Voltmeter
					clear_say();
					sprintf(output, "%.2f", voltage);
					add_num(output);
					add_say(Volts_e);
					break;

				case 2: //Logic
					clear_say();
					if(voltage < LOGIC_THRESHOLD)
						add_say(Beeplow_e);
					else
					{
						add_say(Beephigh_e);
					}
					break;

				case 3: //Min/Max/Mean
				     //get the sample  
					Ain = ADCL;
					Ain |= (ADCH<<8);
					//start another conversion
					ADCSRA |= (1<<ADSC) ; 
					//results to hyperterm
					voltage = (float)Ain / (1024.0/VREF);

					mean = voltage;
					min = voltage;
					max = voltage;
					meanCount = 1;
						
					break;

				case 4:
					clear_say();
					//add_say(Ready_e);
					//add_say(4);
					break;
			}
			buttonDown0 = 2;
		}
		else //Buttondown0 >= 2
		{
			switch(metermode) //Button is held down
			{
				case 1: //voltmeter
				case 2: //Logic
					break; //Do Nothing
				case 3://Min/Max/Mean
				     //get the sample  
					Ain = ADCL;
					Ain |= (ADCH<<8);
					//start another conversion
					ADCSRA |= (1<<ADSC) ; 
					//results to hyperterm
					voltage = (float)Ain / (1024.0/VREF);
					if(voltage > max)
						max = voltage;
					else if(voltage < min)
						min = voltage;
				
					mean = ((mean*(float)meanCount) + voltage)/(float)(++meanCount);
					break;
 				case 4:
					break;
			}
			buttonDown0 = 2;
		}
	}else 
	{
		//LCDsendChar('-');
		if(buttonDown0 == 2)
			buttonDown0 = 3;
		else if(buttonDown0 == 3) //button has been let go
		{
			switch(metermode) //Button has just been let go
			{
				case 1: //voltmeter
				case 2: //Logic
					break; //Do Nothing
				case 3: //Min/Max/Mean
					clear_say();
					add_say(Low_e);					//Low
					sprintf(output, "%.2f", min);
					add_num(output);				//(min)
					add_say(Volts_e);				//Volts
					add_say(High_e);				//High
					sprintf(output, "%.2f", max);
					add_num(output);				//(max)
					add_say(Volts_e);				//Volts
					add_say(Mean_e);				//Mean
					sprintf(output, "%.2f", mean);
					add_num(output);				//(mean)
					add_say(Volts_e);				//Volts
					add_say(Over_e);				//Over
					sprintf(output, "%d", meanCount);
					add_num(output);				//(count)
					add_say(Samples_e);				//Samples
					break;
 				case 4:
					if(frequencyMeasured == 2)
					{	
						clear_say();
						sprintf(output, "%lu", frequency);
						add_num(output);
						add_say(Hz_e);
					}else{
						clear_say();
						add_say(Nosignal_e);
					}
					break;
				default:
					clear_say();
					add_say(Breakfast_e);
					break;
			}

			buttonDown0 = 0;
		}else
			buttonDown0 = 0;
	}

	//Function 4
	//very basic anti-bounce, as we only care about the rising edge
	if(!(PINB & 0x80)) //button is down
	{
		if(buttonDown4 == 0) //and not down before
			buttonDown4++;
		else if(buttonDown4 == 1) //and down once
		{
			metermode = 4;
			buttonDown4 = 2;
		}
	}else 
	{
		buttonDown4 = 0;
	}

	//Function 3
	//very basic anti-bounce, as we only care about the rising edge
	if(!(PINB & 0x40)) //button is down
	{
		if(buttonDown3 == 0) //and not down before
			buttonDown3++;
		else if(buttonDown3 == 1) //and down once
		{
			metermode = 3;
			buttonDown3 = 2;
		}
	}else 
	{
		buttonDown3 = 0;
	}

	//Function 2
	//very basic anti-bounce, as we only care about the rising edge
	if(!(PINB & 0x20)) //button is down
	{
		if(buttonDown2 == 0) //and not down before
			buttonDown2++;
		else if(buttonDown2 == 1) //and down once
		{
			metermode = 2;
			buttonDown2 = 2;
		}
	}else 
	{
		buttonDown2 = 0;
	}

	//Function 1
	//very basic anti-bounce, as we only care about the rising edge
	if(!(PINB & 0x10)) //button is down
	{
		if(buttonDown1 == 0) //and not down before
			buttonDown1++;
		else if(buttonDown1 == 1) //and down once
		{
			metermode = 1;
			buttonDown1 = 2;
		}
	}else 
	{
		buttonDown1 = 0;
	}
}

// --- external interrupt ISR ------------------------
// Triggered on rising edge of frequency detector
uint64_t period;
uint16_t timerTemp;
ISR (INT1_vect) {
	timerTemp = TCNT1;
	TCNT1 = 0;			//begin timing next period
	if(metermode == 4) //we only care about this interupt if it's in frequency mode
	{
		if(frequencyMeasured == 0)
			frequencyMeasured = 1;
		else if(takeFMeasure)
		{
			period = overflowCount;
			period = period << 16;
			period |= timerTemp;
			//period = timerTemp ;//| (overflowCount << 16); //get period (in clocks)
			frequency = (uint32_t)((uint64_t)F_CPU/period);
			frequencyMeasured = 2;
			takeFMeasure = 0;
		}
		overflowCount = 0;
	}
}

// --- Timer 1 Overflow ISR ------
// 
ISR (TIMER1_OVF_vect) {
    overflowCount++;
	TCNT1 = 0;
	if(metermode == 4) //we only care about this interupt if it's in frequency mode
	{
		if(overflowCount >= 40)
			frequencyMeasured = 0;
	}
}
	
int main(void)
begin 

	//set up INT1 for frequency detection
	EIMSK = 1<<INT1 ; // turn on int1
	EICRA = 3<<ISC10 ;       // rising edge

	PORTB = 0x00;
	DDRB = 0x00;
	DDRB=(1<<PORTB3) | (0<<PORTB0) | (0<<PORTB7) | (0<<PORTB6) | (0<<PORTB5) | (0<<PORTB4); 
	// turn on pwm with period= 256 cycles 
	// (62,500 samples/sec) in fast PWM mode.
	// BUT OCR0A update is done using timer2 at 7800/sec
	// timer 0 runs at full rate set in MAIN loop; TCCR0B = 1 ;
	// turn on fast PWM and OC0A output
	// 16 microsec per PWM cycle sample time
	TCCR0A = (1<<COM0A0) | (1<<COM0A1) | (1<<WGM00) | (1<<WGM01) ; 
	OCR0A = 128 ; // set PWM to half full scale

	// turn on timer2 set to overflow at 7812 Hz 
	// (prescaler set to divide by 8)
	TCCR2B = 2;
	// turn on overflow interrupt
	TIMSK2 = (1<<TOIE2);   

	//Enable Timer 1 for frequency count
	TCCR1B = (1<<CS10); //full frequency
	TIMSK1 = (1<<TOIE1); //enable overflow interrupt

	//init the A to D converter 
	//channel zero/ left adj / VCC Vref
	//!!!CONNECT Aref jumper!!!!
	ADMUX = (0<<ADLAR) | (1<<REFS0);  

	//enable ADC and set prescaler to 1/128*16MHz=125,000
	//and clear interupt enable
	//and start a conversion
	ADCSRA = (1<<ADEN) | (1<<ADSC) + 7 ; 

	init_lcd();

	sei();

	add_say(Ready_e);

	while(1) 
	begin  

		switch(metermode) //low frequency updates
		{
			case 1: //voltmeter
			     //get the sample  
				Ain = ADCL;
				Ain |= (ADCH<<8);
				//start another conversion
				ADCSRA |= (1<<ADSC) ; 
				//results to hyperterm
				voltage = (float)Ain / (1024.0/VREF);
				sprintf(output, "%.2f Volts",voltage);

				LCDclr();
				LCDGotoXY(0,0);
				LCDstring(output, strlen(output));
				break;

			case 2: //Logic Probe
			     //get the sample  
				Ain = ADCL;
				Ain |= (ADCH<<8);
				//start another conversion
				ADCSRA |= (1<<ADSC) ; 
				//results to hyperterm
				voltage = (float)Ain / (1024.0/VREF);
				sprintf(output, "Logic: %s", (voltage < LOGIC_THRESHOLD) ? "Low" : "High");

				LCDclr();
				LCDGotoXY(0,0);
				LCDstring(output, strlen(output));
				break;

			case 3:		
					//measurements handled by button code
				LCDclr();
				LCDGotoXY(0, 0);
				sprintf(output, "Mn %.2f  Mx %.2f", min, max);
				LCDstring(output, strlen(output));
				LCDGotoXY(0, 1);
				sprintf(output, "Avg %.2f Num %d", mean, meanCount);
				LCDstring(output, strlen(output));				
				break;

			case 4:		
				if(frequencyMeasured == 2)
				{	
					sprintf(output, "F: %luHz", frequency);

					LCDclr();
					LCDGotoXY(0,0);
					LCDstring(output, strlen(output));
				}else{
					sprintf(output, "F: No Signal");

					LCDclr();
					LCDGotoXY(0,0);
					LCDstring(output, strlen(output));
				}

				break;
		}

		checkButtons();

		_delay_ms(20);

		checkButtons();
		
		_delay_ms(20);

		checkButtons();
		
		_delay_ms(20);

		checkButtons();
		
		_delay_ms(20);

		checkButtons();
		
		_delay_ms(20);

		checkButtons();

		_delay_ms(20);

		checkButtons();
		
		_delay_ms(20);

		checkButtons();
		
		_delay_ms(20);

		checkButtons();
		
		_delay_ms(20);

		checkButtons();

		takeFMeasure++;
		
		_delay_ms(20);
           
	end // end while  
end  //end main
      


